#!/usr/bin/python3

import psycopg2, subprocess, os, sys, random
import numpy as np
from random import randint 
from datetime import datetime, timedelta

host = 'localhost'
ourDB = 'cuqbank'

'''
Ma,Mb,Mc : marks for A, B and C types
Na, Nb, Nc : Number of questions to be answered
Ta, Tb, Tc : Number of questions given  
'''
# CU - Calicut Univ
qpdataM70CU = {'Ma':3,'Mb':6,'Mc':10,'Na':8,'Nb':6,'Nc':1,'Ta':10,'Tb':8,'Tc':2,'DUR':120}
qpdataM50CU = {'Ma':2,'Mb':6,'Mc':10,'Na':8,'Nb':4,'Nc':1,'Ta':10,'Tb':5,'Tc':2,'DUR':90}
# Kannur Univ
qpdataM70KU = {'Ma':3,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':2,'Ta':8,'Tb':6,'Tc':3,'DUR':120}
qpdataM50KU = {'Ma':2,'Mb':6,'Mc':14,'Na':6,'Nb':4,'Nc':1,'Ta':8,'Tb':6,'Tc':2,'DUR':90}

# Models below are for KU departments only
qpdataM32 = {'Ma':1,'Mb':3,'Mc':6, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM40 = {'Ma':2,'Mb':3,'Mc':7, 'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM48 = {'Ma':1,'Mb':5,'Mc':9, 'Na':6,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM60 = {'Ma':3,'Mb':5,'Mc':10,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM80A ={'Ma':4,'Mb':6,'Mc':14,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM80B ={'Ma':4,'Mb':7,'Mc':13,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}
qpdataM80C ={'Ma':4,'Mb':8,'Mc':12,'Na':5,'Nb':3,'Nc':3,'Ta':6,'Tb':5,'Tc':5,'DUR':180}

'''
Fields of the Course Table (Postgres RDBMS)
qtext  		VARCHAR   | Question in LaTex
atext  		VARCHAR   | Answer etc. in LaTex
author 		VARCHAR   | Author info
partnum 	int       | Part Number (Course material divided into 8 parts)
qtype 		CHAR(1)   | Question Type A: Short, B: medium, C: essay type
qnum 		int       | Question number, starts from 1, maximum is 99 (no gaps)
relations 	VARCHAR   | Specify other questions having overlap with this one
mark 		int       | Mark 
status 		VARCHAR   | medium, easy, tough or inactive.
figname 	VARCHAR   | Name of the figure, if any.
figure 		bytea     | Figure data

Each record fetched from the DB is stored into a Python list.
'''

QTEXT = 0		# indices for the fields stored in the list
QATEXT = 1
QAUTHOR = 2
QPART = 3
QTYPE = 4
QSN = 5
QRELATIONS = 6
QMARK = 7
QSTAT = 8
QFIGNAME = 9
QFIG = 10
QUSED = 11    # added by this program
QPART2 = 12   # added by this program

SINGLEQL = 12	# length of the list after adding the 'used' flag

NPART = 8   # Total course matter is divided in to 8 parts, to ensure uniformity


class Course:
	# Arrays to store the number of questions in each Part
	naq = None		
	nbq = None
	ncq = None
	# 2D Arrays to store the questions from each Part
	Atypes = None
	Btypes = None
	Ctypes = None
	#Lists to store the selected questions
	AQ = []
	BQ = []
	CQ = []	

	def __init__(self):
		pass
		
	def findCourse(self, courseCode):
		'''
		Fetches the "coursedetails" Table from Postgres and searches for the
		specified course.
		'''
		conn = psycopg2.connect(database=ourDB, host=host, user='dummy', password="dum123")
		cursor = conn.cursor()
		cursor.execute('select * from coursedetails;')
		res = cursor.fetchall()
		found = False
		for row in res:
			if row[0] == courseCode:
				found = True
				self.courseCode = row[0]
				self.courseName = row[1]
				self.subject = row[2]
				self.degree = row[3]
				self.syllabusYear = row[4]
				self.semester = row[5]
				self.language = row[6]
				self.qpModel = row[7]
				self.duration = row[8]		

		if found == False:
			print('Course Code not found')
			sys.exit('Error')	
		return True

	def fetchData(self):
		'''
		Fetches the question records of the specified course from the Postgres RDBMS and stores them
		to 2D(NPART x Number of questions) arrays Atypes, Ntypes and Ctypes.
		Stores the number of questions info in arrays naq, nbq and ncq.
		'''
		try:
			conn = psycopg2.connect(database=ourDB, host=host, user=self.courseCode.lower(), password = passwd)
			cursor = conn.cursor()	
			cursor.execute("SELECT * FROM %s"%courseCode)	
			results = cursor.fetchall()
		except:
			print('Connection to Database "%s" failed. Check password.'%ourDB)
			sys.exit('Database connection failed')
			
		# Count the number of questions in each section, type wise
		self.naq = np.zeros(NPART, dtype=int)
		self.nbq = np.zeros(NPART, dtype=int)
		self.ncq = np.zeros(NPART, dtype=int)

		for row in results:
			for k in range(NPART):		# k goes from 0 to NPART-1. Add 1 while comparing
				if row[QTYPE] == 'A' and row[QPART] == k+1:
					self.naq[k] += 1
				if row[QTYPE] == 'B' and row[QPART] == k+1:
					self.nbq[k] += 1
				if row[QTYPE] == 'C' and row[QPART] == k+1:
					self.ncq[k] += 1
		
		#print('NQs ',self.naq, self.nbq, self.ncq)
		# Make 3 array of objects, size NPART, to transfer the SQL query results
		self.Atypes = np.empty(NPART,object)
		self.Btypes = np.empty(NPART,object)
		self.Ctypes = np.empty(NPART,object)

		for k in range(NPART):	# Array size = NPART x NoQ in each (variable)
			self.Atypes[k] = np.empty(self.naq[k], object)
			self.Btypes[k] = np.empty(self.nbq[k], object)
			self.Ctypes[k] = np.empty(self.ncq[k], object)

		for k in range(len(results)):	# Copy to 3 2D Array of objects
			row = list(results[k])
			row.append(False)	# Add the 'QUSED' flag, set False			
			if row[QSTAT] == 'inactive':
				row[QUSED] = True
			if row[QSTAT] == 'easy' and qpLevel == 'tough':
				row[QUSED] = True
			if row[QSTAT] == 'tough' and qpLevel == 'easy':
				row[QUSED] = True
			if row[QTYPE] == 'A':
				sec = row[QPART]
				qn = row[QSN]
				self.Atypes[sec-1][qn-1] = row	# Indices from 0 to NPART-1 (not 1 to NPART)
			if row[QTYPE] == 'B':
				sec = row[QPART]
				qn = row[QSN]
				self.Btypes[sec-1][qn-1] = row
			if row[QTYPE] == 'C':
				sec = row[QPART]
				qn = row[QSN]
				self.Ctypes[sec-1][qn-1] = row
			if row[QFIGNAME] != None:			# Save is there is a figure
				f = open(row[QFIGNAME],'wb')
				f.write(row[QFIG])
				f.close()	
		print('Fetched %d Questions'%len(results))		
		
	def selectM70KU(self):
		'''
		Ta = 8, Tb = 6, Tc = 3 
		Code below selects the Part (0 to 7) from which each question will be taken, and puts them in three lists ia, ib and ic.
		'''
		ic = []
		ic.append(randint(0, NPART -1))
		if ic[0] % 2:  		# odd number
			ic.append(ic[0] - 1)
		else:
			ic.append(ic[0] + 1)
		while 1:
			n = randint(0,NPART-1)
			if n not in ic:
				ic.append(n)
				break
		ic3 = [ic[2]]   # 2 Parts from which the third essay is picked
		if ic[2] % 2:  		
			ic3.append(ic[2] - 1)
		else:
			ic3.append(ic[2] + 1)			
				
		ib = []
		for k in range(NPART):
			if k not in ic + ic3: 
				ib.append(k)
				if k%2 == 0:
					ib.append(k+randint(0,1))
		ia = []
		for k in range(NPART):
			if k not in [ic[0], ic[1]]:
				ia.append(k)
		for k in range(0,NPART,2):		
				if k not in ic[:2] and k not in ic3:
					ia.append(k + randint(0,1))
		for k in range(NPART):
			if ia[k] == ic3[1]:     # move A type to the same Part # of selected C
				ia[k] = ic3[0]			
		return ia,ib,ic

	def selectM50KU(self):  # For 50 marks, Parts to pick A,B and C types.
		'''
		Ta = 8, Tb = 6, Tc = 2 
		Code below decides the Part (0 to 5) from which each question will be selected, and puts them in three lists ia, ib and ic.
		'''
		NPART50 = 6			# Only the first Six Parts
		ic = []
		ic.append(randint(0, NPART50-1))	 # pick from 6 Parts only
		if ic[0] % 2:  		# odd number
			ic.append(ic[0] - 1)
		else:
			ic.append(ic[0] + 1)

		ib = []
		for k in range(NPART50):
			if k not in ic:
				ib.append(k)
		for k in range(0,NPART50,2):		
				if k not in ic:
					ib.append(k + randint(0,1))
		
		ia = []
		for k in range(NPART50):
			if k not in ic:
				ia.append(k)
				ia.append(k)
		return ia,ib,ic
		
	def selectM70CU(self):  # Univ. of Calicut , 70 marks paper
		# Ta = 10, Tb = 8, Tc = 2 
		ic = []
		ic.append(randint(0, NPART -1))  # First C type
		while 1:
			n = randint(0,NPART-1)   # Second C type
			if abs(ic[0] - n) > 1:
				ic.append(n)
				break

		ib = []
		for k in range(NPART):
			if k not in ic: 
				ib.append(k)
		while True:
			n = randint(0,NPART-1)  
			if n not in ic: 
				ib.append(n)
			if len(ib) == 8:
				break
		
		ia = []
		for k in range(NPART):
			ia.append(k)   # 8 questions
		while True:		
			n = randint(0,NPART-1)
			if n not in ic:
				ia.append(n)
			if len(ia) >= 10:
				break
		return ia,ib,ic

	def selectM50CU(self):  # Ta = 8, Tb = 6, Tc = 2 
		ic = []
		ic.append(randint(0, NPART -1))  # First C type
		while 1:
			n = randint(0,NPART-1)   # Second C type
			if abs(ic[0] - n) > 1:
				ic.append(n)
				break
		
		ib = []
		while True:
			n = randint(0,NPART-1)  
			if n not in ic+ib: 
				ib.append(n)
			if len(ib) == 5:
				break
			
		ia = []
		for k in range(NPART):
			ia.append(k)   # 8 questions
			if k not in ib and k not in ic:
				ia.append(k)
		while True:		
			n = randint(0,NPART-1)
			if n not in ic:
				ia.append(n)
			if len(ia) == 10:
				break
		return ia,ib,ic

	def selectUQ(self):
		'''
		For Kannur Univ. departments. Select the Parts to pick A,B and C type questions.
		Ta = 6, Tb = 5, Tc = 5 
		'''
		ic = []
		for k in range(0,NPART,2):
			ic.append(randint(0,1)+k)
		while 1:
			n = randint(0,NPART-1)
			if n not in ic:
				ic.append(n)
				break
		ib = []
		for p in range(NPART):
			if p not in ic:
				ib.append(p)
		random.shuffle(ib)
		
		ia = ic.copy() 			# Five A types  from the same sections
		ia.append(ib[-1])		# Add third section of B to A	
		ib = (ib*2)[:-1]		# repeat the first two sections of B type 
		return ia,ib,ic	

	def markRelations(self,part,relation): # Mark the specified questions as used
		if relation == '':
			return
		rels = relation.split(',')
		for r in rels:
			r = r.strip()
			rtype = r[0]
			rval = r[1:]
			rqn = eval(r[1:])-1
			if rtype == 'C':
				self.Ctypes[part][rqn][QSTAT] = True
			if rtype == 'A':
				self.Atypes[part][rqn][QUSED] = True

	def showDist(self, ia,ib,ic):
		pat = '<br><br>Sectionwise distribution of the questions are shown below.<br>'
		for k in range(NPART):
			pat += 'P%d('%(k+1)
			if k in ic: pat += 'C'
			if k in ib: pat += 'B'
			if k in ia: pat += '%-2s) '%('A'*ia.count(k))
		pat += '<br>'
		print(pat)
		f = open('pat.txt','w')
		f.write(pat)	
			
	def makeQP(self):
		if self.qpModel == 'M50KU':
			self.qpd = qpM50KU
			# Check whether there is enough number of C types in the selected Part #
			for k in range(5):		# Five trials maximum
				ia,ib,ic = self.selectM50KU()	
				#print (ic, self.ncq[ic[0]], self.ncq[ic[1]])
				if self.ncq[ic[0]] > 1 and self.ncq[ic[1]] > 1:
					break						
		elif self.qpModel == 'M70KU':
			self.qpd = qpM70KU
			# Check whether there is enough number of C types in the selected Part #
			for k in range(5):		# Five trials maximum
				ia,ib,ic = self.selectM70KU()	
				#print (ic, self.ncq[ic[0]], self.ncq[ic[1]], self.ncq[ic[2]])
				if self.ncq[ic[0]] > 1 and self.ncq[ic[1]] > 1 and self.ncq[ic[2]] > 1:
					break
		elif self.qpModel == 'M70CU':
			self.qpd = qpdataM70CU
			for k in range(5):		# Five trials maximum
				ia,ib,ic = self.selectM70CU()	
				#print (ic, self.ncq[ic[0]], self.ncq[ic[1]])
				if self.ncq[ic[0]] > 1 and self.ncq[ic[1]] > 1:
					break						
		elif self.qpModel == 'M50CU':
			self.qpd = qpdataM50CU
			for k in range(5):		# Five trials maximum
				ia,ib,ic = self.selectM50CU()	
				#print (ic, self.ncq[ic[0]], self.ncq[ic[1]])
				if self.ncq[ic[0]] > 1 and self.ncq[ic[1]] > 1:
					break						

			ia,ib,ic = self.selectM50CU()	
		elif self.qpModel == 'M60':  # KU department only
			self.qpd = qpdataM60
			ia,ib,ic = self.selectUQ()	
		else:	
			sys.exit('Error: Unsupported Question paper type.')	
		ia.sort()		
		ib.sort()
		ic.sort()

		#print ('IABC = ', ia, ib, ic)
		self.showDist(ia, ib, ic)
		
		self.MAXMARK = self.qpd['Ma'] * self.qpd['Na'] + self.qpd['Mb'] * self.qpd['Nb'] +\
		self.qpd['Mc'] * self.qpd['Nc']
		for k in range(len(ic)):		# Select Ctype first
			part = ic[k]				# Pre-selected Part #s are stored in ic
			random.shuffle(self.Ctypes[part])
			for q in self.Ctypes[part]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.CQ.append(q)
					#print('First C: ', part, q[QSN], q[QMARK])
					self.markRelations(part, q[QRELATIONS])
					if	q[QMARK] != self.qpd['Mc']:		# get one more piece 
						mr2 = self.qpd['Mc'] - q[QMARK]
						#print ('CM mr mr2 = ', CMARK, q[QMARK],mr2)
						random.shuffle(self.Ctypes[part])
						for q in self.Ctypes[part]:
							if q[QMARK] == mr2 and q[QUSED] == False:
								q[QUSED] = True
								self.CQ[-1].append(q)	# next piece
								#print('Second C: ', part, q[QSN], q[QMARK])
								self.markRelations(part, q[QRELATIONS])
								break
					break
					
		for k in range(len(ib)):	# Select Ctype first
			part = ib[k]			
			random.shuffle(self.Btypes[part])		
			for q in self.Btypes[part]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.BQ.append(q)
					self.markRelations(part, q[QRELATIONS])
					break
		
		for k in range(len(ia)):	# Select A type
			part = ia[k]			
			random.shuffle(self.Atypes[part])
			for q in self.Atypes[part]:
				if q[QUSED] == False:
					q[QUSED] = True
					self.AQ.append(q)
					self.markRelations(part, q[QRELATIONS])
					break

		#Verify the results.
		if len(self.AQ) != len(ia):
			print('Could get only %d number of A type questions. Requires %d'%(len(self.AQ), len(ia)))
			sys.exit(1)
		if len(self.BQ) != len(ib):
			print('Could get only %d number of B type questions. Requires %d'%(len(self.BQ), len(ib)))
			sys.exit(1)
		if len(self.CQ) != len(ic):
			print('Could get only %d number of C type questions. Requires %d'%(len(self.CQ), len(ic)))
			sys.exit(1)
		
		for q1 in self.CQ:
			if q1[QMARK] != self.qpd['Mc']  and len(q1) == SINGLEQL:
				print('C type part1 %d marks. Could not find second part'%(q1[QMARK]))
				sys.exit(1)
		# End of make QP	
		
	def makeTexFiles(self):
		now = datetime.now() + timedelta(hours = 5, minutes=30)
		self.dt = now.strftime("%d/%m/%Y %H:%M:%S")

		#print('Python: Answer Done. Making QP..')
		#--------Generate Latex document, using data from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document}\n'
		
		tex += '\\begin{center}\n'
		tex += 'Reg No:.................. \\hfill \\small {%s}\n\n'%(qpCode)
		tex += 'Name :.................... \\hfill  \\small {%s}\n\n'%('')	
		
		tex += '\\large\n \\textbf {%s Semester %s %s Examination\n'%(self.semester, self.degree, self.subject)
		tex += '\\\\%s %s\n'%(examMonth, examYear)
		courseYear = self.courseCode[1:5]
		tex += '(%s Admission onwards)\\\\\n'%courseYear # syllabusYear
		tex += '%s (%s)}\n\n'%(self.courseCode[5:],self.courseName)
		tex += '(%s)\n\n'%qpHeader
		tex += '\\vspace{0.1in}\n\\end{center}\n '
		tex += '\\hrule\n\\vspace{0.1in}\n'
	
		tex += '\\noindent\n' 
		tex += 'Time : %3d min  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		
		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])

		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }\n'%(self.qpd['Nb'], self.qpd['Mb'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
		
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])
		
		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'		
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d] %s \\droppoints\n'%(q[QMARK],q[QTEXT].strip())
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex += '\\part[%d] %s \\droppoints'%(q[QMARK], q[QTEXT].strip())
				tex += '\\part[%d] %s \\droppoints'%(q2[QMARK], q2[QTEXT].strip())
				tex += '\\end{parts}'	
				tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\end{questions}\n'
		tex += '\\end{document}'

		fn = courseCode
		open(fn + '.tex', 'w').write(tex)
		#print('Python: Written Question .tex file ',fn)
		self.answerTex()

	def answerTex(self):
		#print('Python: Entering answerTex..')
		#--Generate Latex document, using questions & answers from AQ,BQ and CQ -----
		tex = '\\documentclass[12pt, a4paper]{exam}\n \\usepackage{graphicx}\n \\usepackage{amsmath}\n \\usepackage{amssymb}\n'
		tex += '\\usepackage[left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'

		if self.language.lower() == 'malayalam':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Malayalam]{Rachana}'			
		elif self.language.lower() == 'hindi':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'		
		elif self.language.lower() == 'arabic':
			tex += '\\usepackage{fontspec}\n\\usepackage{polyglossia}\n'
			tex += '\\setotherlanguage{arabic}\n'
			tex += '\\setmainfont[Script=Arabic, Scale=1.1]{Scheherazade}\n'	
			tex += '\\usepackage{bidi}\n\\newcommand{\PRL}\n\n' 

		tex += '\\begin{document} \\noindent\n'

		tex += '%s \\hfill %s\n\\newline '%(qpCode, self.dt)
		
		tex += '\\begin{center} \\large\n \\textbf {%s Semester %s %s Examination\n'%(self.semester, self.degree, self.subject)
		tex += '%s %s\n'%(examMonth, examYear)
		tex += '(%s Admission onwards)\\\\\n'%self.syllabusYear
		tex += '%s (%s)}\n\\end{center}\n'%(self.courseCode, self.courseName)
		
		tex += '\\noindent\n' 
		tex += 'Time : %s  \hfill Maximum Marks : %2d\n\\vspace{0.1in}\n\n'%(self.qpd['DUR'], self.MAXMARK)
		tex += '\\hrule\n\\vspace{0.1in}\n'

		tex += '{\\noindent \\bfseries  Part A (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Na'], self.qpd['Ma'])
		tex += '\\begin{questions} \\pointsdroppedatright\n'

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.AQ:
			tex += '\\question[%d]P%d(%d):%s\n\n'%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip())
			
			if self.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '{\\noindent \\small \\bfseries  Part B (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nb'], self.qpd['Mb'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.BQ:
			tex += '\\question[%d]P%d(%d):%s\n\n'%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip())
			
			if self.language.lower() == 'arabic':
				tex += '\\textbf{إجابة}\n\n'
			else:
				tex += '\\textbf{Answer:}\n\n'
			
			tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
			tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'
			
		tex += '{\\noindent\n\\small \\bfseries Part C (Answer any %d questions. Each carries %d marks) }'%(self.qpd['Nc'], self.qpd['Mc'])

		if self.language.lower() == 'arabic':
			tex += '\\begin{Arabic}\n'
		for q in self.CQ:
			if len(q) == SINGLEQL:
				tex += '\\question[%d]P%d(%d):%s\n\n'%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip())
				
				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'
				
				tex += '%s\n\\droppoints\n'%q[QATEXT].strip()
				tex += '\\vspace{0.05in}\n'
			else:
				q2 = q[QPART2]
				tex += '\\question\n'
				tex += '\\begin{parts} \n'
				tex +='\\part[%d]P%d(%d):%s\n\n'%(q[QMARK],q[QPART],q[QSN],q[QTEXT].strip())

				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q[QATEXT].strip()				
				tex += '\\part[%d]P%d(%d): %s\n\n'%(q2[QMARK],q2[QPART],q2[QSN],q2[QTEXT].strip())
				
				if self.language.lower() == 'arabic':
					tex += '\\textbf{إجابة}\n\n'
				else:
					tex += '\\textbf{Answer:}\n\n'

				tex += '%s\\droppoints\n'%q2[QATEXT].strip()
				tex += '\\end{parts}'	
				tex += '\\vspace{0.05in}\n'
		if self.language.lower() == 'arabic':
			tex += '\\end{Arabic}\n'

		tex += '\\end{questions}\n'
		tex += '\\end{document}'

		fn = courseCode + '_ANS'
		open(fn + '.tex', 'w').write(tex)
		#print('Python: Written Answer .tex file ',fn)


#---- main program -----

ss = open('info.txt').read().split('\n')
#os.unlink('info.txt')
for row in ss:
	sss = row.split(':')
	#print(sss)
	if sss[0] == 'examYear':
		examYear = sss[1]
	elif sss[0] == 'examMonth':
		examMonth = sss[1]
	elif sss[0] == 'courseCode':
		courseCode = sss[1]
	elif sss[0] == 'passwd':
		passwd = sss[1]
	elif sss[0] == 'qpCode':
		qpCode = sss[1]
	elif sss[0] == 'header':
		qpHeader = sss[1]
	elif sss[0] == 'level':
		qpLevel = sss[1]

#print(courseCode, passwd, examYear, examMonth, qpCode, qpHeader, qpLevel)

cc = Course()
cc.findCourse(courseCode)
cc.fetchData()	
cc.makeQP()
cc.makeTexFiles()

print('Done')
sys.exit('Done')

